#ifndef ERR_H
#define ERR_H

#include <stdio.h>	/* vsnprintf(3) */
#include <stdarg.h>	/* va_list va_start va_end */
#include <stdlib.h>	/* exit(3) */

#include <errno.h>

/*
 * Fake err(3) and errx(3) from err.h since Solaris doesn't provide it.
 */
static void err(int retcode, const char *fmt, ...) {
	int sys_errno	= errno;
	char buf[256];
	va_list ap;

	va_start(ap, fmt);

	(void)vsnprintf(buf, sizeof buf, fmt, ap);

	va_end(ap);

	(void)fprintf(stderr, "%s: %s\n", buf, strerror(sys_errno));

	exit(retcode);
} /* err() */


static void errx(int retcode, const char *fmt, ...) {
	int sys_errno	= errno;
	char buf[256];
	va_list ap;

	va_start(ap, fmt);

	(void)vsnprintf(buf, sizeof buf, fmt, ap);

	va_end(ap);

	(void)fprintf(stderr, "%s\n", buf);

	exit(retcode);
} /* errx() */


static void warn(const char *fmt, ...) {
	int sys_errno	= errno;
	char buf[256];
	va_list ap;

	va_start(ap, fmt);

	(void)vsnprintf(buf, sizeof buf, fmt, ap);

	va_end(ap);

	(void)fprintf(stderr, "%s: %s\n", buf, strerror(sys_errno));

	fflush(stderr);

	return /* void */;
} /* err() */


static void warnx(const char *fmt, ...) {
	int sys_errno	= errno;
	char buf[256];
	va_list ap;

	va_start(ap, fmt);

	(void)vsnprintf(buf, sizeof buf, fmt, ap);

	va_end(ap);

	(void)fprintf(stderr, "%s\n", buf);

	fflush(stderr);

	return /* void */;
} /* errx() */

#endif
